package tycho.surefire.problem.chart.tests;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.birt.chart.model.ChartWithAxes;
import org.eclipse.birt.chart.model.attribute.ColorDefinition;
import org.eclipse.birt.chart.model.attribute.impl.ColorDefinitionImpl;
import org.eclipse.birt.chart.model.component.Axis;
import org.eclipse.birt.chart.model.data.DateTimeDataSet;
import org.eclipse.birt.chart.model.data.SeriesDefinition;
import org.eclipse.birt.chart.model.type.LineSeries;
import org.eclipse.birt.core.framework.Platform;
import org.eclipse.birt.core.framework.PlatformConfig;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.junit.Assert;
import org.junit.Test;

import tycho.surefire.problem.chart.ChartComposite;
import tycho.surefire.problem.chart.Charts;
import tycho.surefire.problem.chart.LinesChart;
import tycho.surefire.problem.chart.LinesChart.LineData;
import tycho.surefire.problem.chart.RandomUtils;

public class TestLinesChart {
	
	public static void temporaryOpen(final Shell shell) {
		new Thread() {
			@Override
			public void run() {
				try {
					Thread.sleep(500);
				} catch (final InterruptedException e) {
					// ignore
				}
				Display.getDefault().syncExec(new Runnable() {
					@Override
					public void run() {
						if (!shell.isDisposed()) {
							shell.close();
						}
					}
				});
			}
		}.start();
		
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}
	}
	
	private static final ColorDefinition[]	COLORS		= { ColorDefinitionImpl.RED(),
														ColorDefinitionImpl.BLUE(),
														ColorDefinitionImpl.GREEN(),
														ColorDefinitionImpl.YELLOW()
														};
	private static final LineData			lineData1	= new LineData("line1", TestLinesChart.COLORS[0], new Double[] { 0d, 1d, 2d, 3d, 4d, 5d, 6d, 7d, 8d, 9d }, true);				//$NON-NLS-1$
	private static final LineData			lineData2	= new LineData("line2", TestLinesChart.COLORS[1], new Double[] { 10d, 11d, 12d, 13d, 14d, 15d, 16d, 17d, 18d, 19d }, false);	//$NON-NLS-1$
	private static final LineData			lineData3	= new LineData("line3", TestLinesChart.COLORS[2], new Double[] { 20d, 21d, 22d, 23d, 24d, 25d, 26d, 27d, 28d, 29d }, true);	//$NON-NLS-1$
																																														
	private static final LineData			lineData4	= new LineData("line4", TestLinesChart.COLORS[3], new Double[] { 30d, 31d, 32d, 33d, 34d, 35d, 36d, 37d, 38d, 39d }, false);	//$NON-NLS-1$

	private static final List<LineData>		lines		= Arrays.asList(TestLinesChart.lineData1, TestLinesChart.lineData2, TestLinesChart.lineData3, TestLinesChart.lineData4);
	
	@Test
	public void testConstructor() {
		
		Exception expected = null;
		try {
			new LinesChart(null, Charts.randomDates(9), Arrays.asList(TestLinesChart.lineData1), false, false);
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);
		
		expected = null;
		try {
			new LinesChart("chart", Charts.randomDates(9), Arrays.asList(TestLinesChart.lineData1), false, false); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

		expected = null;
		try {
			new LinesChart("chart", Charts.randomDates(9), null, false, false); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

		expected = null;
		try {
			final List<LineData> data = Collections.emptyList();
			new LinesChart("chart", Charts.randomDates(9), data, false, false); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);
		
	}
	
	@Test
	@SuppressWarnings("deprecation")
	public void testDisplay() {
		
		final LinesChart chart = new LinesChart("chart", Charts.randomDates(10), TestLinesChart.lines, false, false); //$NON-NLS-1$
		
		final Display display = Display.getDefault();
		
		final Shell shell = new Shell(display);
		shell.setSize(600, 400);
		shell.setLayout(new GridLayout());
		
		final PlatformConfig config = new PlatformConfig();
		config.setProperty("STANDALONE", Boolean.valueOf(!Platform.runningEclipse()).toString()); //$NON-NLS-1$
		
		final GridData data = new GridData(SWT.CENTER, SWT.CENTER, true, true);
		data.widthHint = 600;
		data.heightHint = 400;

		final ChartComposite chartCompo = new ChartComposite(shell, chart, config);
		chartCompo.setLayoutData(data);
		
		TestLinesChart.temporaryOpen(shell);
		
	}
	
	@Test
	public void testLineData() {
		
		new LineData("test", ColorDefinitionImpl.RED(), new Double[] { 1d, 2d, 3d }, true); //$NON-NLS-1$
		
		Exception expected = null;
		try {
			new LineData(null, ColorDefinitionImpl.RED(), new Double[] { 1d, 2d, 3d }, true);
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);
		
		expected = null;
		try {
			new LineData("test", null, new Double[] { 1d, 2d, 3d }, true); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);
		
		expected = null;
		try {
			new LineData("test", ColorDefinitionImpl.RED(), null, true); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);
		expected = null;
		try {
			new LineData("test", ColorDefinitionImpl.RED(), new Double[] {}, true); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);
		
	}
	
	@Test
	public void testLinesChart() {
		
		final Boolean stack = RandomUtils.randomBoolean();
		final Boolean showGrid = RandomUtils.randomBoolean();
		
		final LinesChart chart = new LinesChart("chart", Charts.randomDates(10), TestLinesChart.lines, showGrid, stack); //$NON-NLS-1$

		Assert.assertTrue(chart.getTitle().isVisible());
		Assert.assertEquals("chart", chart.getTitle().getLabel().getCaption().getValue()); //$NON-NLS-1$
		Assert.assertNotNull(chart);
		Assert.assertTrue(ChartWithAxes.class.isAssignableFrom(chart.getClass()));
		Assert.assertNotNull(chart.getPrimaryBaseAxes());

		final Axis xAxis = chart.getPrimaryBaseAxes()[0];
		Assert.assertEquals(1, xAxis.getSeriesDefinitions().size());
		Assert.assertEquals(1, xAxis.getSeriesDefinitions().get(0).getSeries().size());
		
		Assert.assertTrue(xAxis.getSeriesDefinitions().get(0).getSeries().get(0).getDataSet() instanceof DateTimeDataSet);
		Assert.assertNotNull(chart.getPrimaryOrthogonalAxis(xAxis));
		
		final Axis yAxis = chart.getPrimaryOrthogonalAxis(xAxis);
		Assert.assertEquals(1, yAxis.getSeriesDefinitions().size());
		
		final SeriesDefinition ySeriesDefinition = yAxis.getSeriesDefinitions().get(0);
		
		Assert.assertEquals(4, ySeriesDefinition.getSeriesPalette().getEntries().size());
		
		for (int i = 0; i < TestLinesChart.COLORS.length; i++) {
			Assert.assertEquals(TestLinesChart.COLORS[i], ySeriesDefinition.getSeriesPalette().getEntries().get(i));
		}
		
		Assert.assertEquals(4, ySeriesDefinition.getSeries().size());
		
		for (int i = 0; i < TestLinesChart.COLORS.length; i++) {
			Assert.assertEquals(TestLinesChart.COLORS[i], ((LineSeries) ySeriesDefinition.getSeries().get(i)).getLineAttributes().getColor());
		}
		
		// TODO add stack & showGrid assert
		
		final LinesChart chart2 = new LinesChart(null, Charts.randomDates(10), TestLinesChart.lines, showGrid, stack);
		
		Assert.assertFalse(chart2.getTitle().isVisible());
		Assert.assertNull(chart2.getTitle().getLabel().getCaption().getValue());

	}
	
}
