package tycho.surefire.problem.chart.tests;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.birt.chart.model.ChartWithoutAxes;
import org.eclipse.birt.chart.model.attribute.ColorDefinition;
import org.eclipse.birt.chart.model.attribute.impl.ColorDefinitionImpl;
import org.eclipse.birt.chart.model.data.SeriesDefinition;
import org.junit.Assert;
import org.junit.Test;

import tycho.surefire.problem.chart.PieChart;
import tycho.surefire.problem.chart.PieChart.PieData;
import tycho.surefire.problem.chart.RandomUtils;

public class TestPieChart {

	private static final ColorDefinition[]	COLORS		= { ColorDefinitionImpl.RED(),
		ColorDefinitionImpl.BLUE(),
		ColorDefinitionImpl.GREEN(),
		ColorDefinitionImpl.YELLOW()
	};

	private static final PieData			pieData1	= new PieData("pie1", RandomUtils.randomDouble(0, 100), TestPieChart.COLORS[0]);								//$NON-NLS-1$
	private static final PieData			pieData2	= new PieData("pie2", RandomUtils.randomDouble(0, 100), TestPieChart.COLORS[1]);								//$NON-NLS-1$
	private static final PieData			pieData3	= new PieData("pie3", RandomUtils.randomDouble(0, 100), TestPieChart.COLORS[2]);								//$NON-NLS-1$
	private static final PieData			pieData4	= new PieData("pie4", RandomUtils.randomDouble(0, 100), TestPieChart.COLORS[3]);								//$NON-NLS-1$

	private static final List<PieData>		pies		= Arrays.asList(TestPieChart.pieData1, TestPieChart.pieData2, TestPieChart.pieData3, TestPieChart.pieData4);

	@Test
	public void testConstructor() {

		Exception expected = null;
		try {
			new PieChart("pie", null); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

		try {
			new PieChart(null, Arrays.asList(TestPieChart.pieData1));
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

		try {
			final List<PieData> data = Collections.emptyList();
			new PieChart("pie", data); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

	}

	@Test
	public void testPieChart() {

		final PieChart chart = new PieChart("chart", TestPieChart.pies); //$NON-NLS-1$

		Assert.assertTrue(chart.getTitle().isVisible());
		Assert.assertEquals("chart", chart.getTitle().getLabel().getCaption().getValue()); //$NON-NLS-1$
		Assert.assertNotNull(chart);
		Assert.assertTrue(ChartWithoutAxes.class.isAssignableFrom(chart.getClass()));
		Assert.assertEquals(1, chart.getSeriesDefinitions().size());

		final SeriesDefinition baseSeriesDefintion = chart.getSeriesDefinitions().get(0);

		Assert.assertEquals(4, baseSeriesDefintion.getSeriesPalette().getEntries().size());

		for (int i = 0; i < TestPieChart.COLORS.length; i++) {
			Assert.assertEquals(TestPieChart.COLORS[i], baseSeriesDefintion.getSeriesPalette().getEntries().get(i));
		}

		Assert.assertEquals(1, baseSeriesDefintion.getSeriesDefinitions().size());

		final SeriesDefinition seriesDefintion = baseSeriesDefintion.getSeriesDefinitions().get(0);
		
		Assert.assertEquals(1, seriesDefintion.getSeries().size());
		
		final PieChart chart2 = new PieChart(null, TestPieChart.pies);

		Assert.assertFalse(chart2.getTitle().isVisible());
		Assert.assertNull(chart2.getTitle().getLabel().getCaption().getValue());

	}

	@Test
	public void testPieData() {

		new PieData("test", Double.valueOf(1d), ColorDefinitionImpl.RED()); //$NON-NLS-1$

		// -------------------- label null

		Exception expected = null;
		try {
			new PieData(null, Double.valueOf(1d), ColorDefinitionImpl.RED());
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

		// -------------------- label blanck

		expected = null;
		try {
			new PieData("", Double.valueOf(1d), ColorDefinitionImpl.RED()); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

		// -------------------- value null

		expected = null;
		try {
			new PieData("test", null, ColorDefinitionImpl.RED()); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

		// -------------------- color null

		expected = null;
		try {
			new PieData("test", Double.valueOf(1d), null); //$NON-NLS-1$
		} catch (final Exception e) {
			expected = e;
		}
		Assert.assertNotNull(expected);

	}

}
