package tycho.surefire.problem.chart;

import org.eclipse.birt.chart.device.ICallBackNotifier;
import org.eclipse.birt.chart.device.IDeviceRenderer;
import org.eclipse.birt.chart.exception.ChartException;
import org.eclipse.birt.chart.factory.GeneratedChartState;
import org.eclipse.birt.chart.factory.Generator;
import org.eclipse.birt.chart.model.Chart;
import org.eclipse.birt.chart.model.attribute.Bounds;
import org.eclipse.birt.chart.model.attribute.CallBackValue;
import org.eclipse.birt.chart.model.attribute.impl.BoundsImpl;
import org.eclipse.birt.chart.util.PluginSettings;
import org.eclipse.birt.core.framework.PlatformConfig;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;

public final class ChartComposite
		extends Composite
		implements PaintListener, ICallBackNotifier, ControlListener {
	
	private final Chart			chart;
	
	private IDeviceRenderer		idr;
	private GeneratedChartState	gcs;
	
	private boolean				needsGeneration	= true;
	
	public ChartComposite(final Composite parent, final Chart chart) {
		this(parent, chart, null);
	}
	
	public ChartComposite(final Composite parent, final Chart chart, final PlatformConfig config) {
		super(parent, SWT.NONE);
		
		this.chart = chart;
		
		try {
			this.idr = PluginSettings.instance(config).getDevice("dv.SWT");//$NON-NLS-1$
			this.idr.setProperty(IDeviceRenderer.UPDATE_NOTIFIER, this);
		} catch (final ChartException ex) {
			ex.printStackTrace();
		}
		
		this.addPaintListener(this);
		this.addControlListener(this);
	}
	
	@Override
	public void callback(final Object event, final Object source, final CallBackValue value) {
		if (this.chart instanceof ICallBackNotifier) {
			((ICallBackNotifier) this.chart).callback(event, source, value);
		}
	}
	
	@Override
	public void controlMoved(final ControlEvent e) {
		// do nothing
	}
	
	@Override
	public void controlResized(final ControlEvent e) {
		this.regenerateChart();
	}
	
	@Override
	public Chart getDesignTimeModel() {
		return this.chart;
	}
	
	@Override
	public Chart getRunTimeModel() {
		return this.gcs.getChartModel();
	}
	
	@Override
	public void paintControl(final PaintEvent e) {
		
		final Rectangle d = this.getClientArea();
		final Image imgChart = new Image(this.getDisplay(), d);
		final GC gcImage = new GC(imgChart);
		this.idr.setProperty(IDeviceRenderer.GRAPHICS_CONTEXT, gcImage);
		
		final Bounds bo = BoundsImpl.create(0, 0, d.width, d.height);
		bo.scale(72d / this.idr.getDisplayServer().getDpiResolution());
		
		final Generator gr = Generator.instance();
		if (this.needsGeneration) {
			try {
				this.gcs = gr.build(this.idr.getDisplayServer(), this.chart, bo, null, null, null);
				this.needsGeneration = false;
			} catch (final ChartException ce) {
				ce.printStackTrace();
			}
		}
		
		try {
			gr.render(this.idr, this.gcs);
			e.gc.drawImage(imgChart, d.x, d.y);
		} catch (final ChartException ce) {
			ce.printStackTrace();
		}
	}
	
	@Override
	public Object peerInstance() {
		return this;
	}
	
	@Override
	public void regenerateChart() {
		this.needsGeneration = true;
		this.repaintChart();
	}
	
	@Override
	public void repaintChart() {
		this.redraw();
	}
	
}
